
# q-expansion of newform 810.3.j.b, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 810
weight = 3
poly_data = [1, 0, 0, 0, -1, 0, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[731, [1, 0, 0, 0, -1, 0, 0, 0]], [487, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0, 0, 0, 0, -1], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 4, 0, -3, 0, -4], [0, 0, 11, 0, 0, 0, -11, 0], [0, 5, 0, 0, 0, 0, 0, -5], [0, 0, -15, 0, 0, 0, 0, 0], [0, 16, 0, 16, 0, -16, 0, 0], [-3, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 13, 0, -13, 0, -13], [0, -9, 0, 0, 0, 0, 0, 9], [8, 0, 0, 0, -8, 0, 0, 0], [0, 0, 0, 0, 0, 0, 65, 0], [0, 0, 0, 55, 0, 55, 0, -55], [0, 0, -32, 0, 0, 0, 32, 0], [0, 40, 0, 0, 0, 0, 0, 40], [0, 9, 0, 9, 0, -9, 0, 0], [0, 0, 0, -56, 0, -56, 0, 56], [0, 0, 0, 0, -95, 0, 0, 0], [0, 0, -19, 0, 0, 0, 0, 0], [0, 3, 0, -3, 0, -3, 0, 0], [0, 0, 0, 0, 0, 0, -119, 0], [0, 0, 0, 0, -99, 0, 0, 0], [0, -77, 0, 0, 0, 0, 0, -77], [0, -64, 0, 64, 0, 64, 0, 0], [0, 0, -95, 0, 0, 0, 95, 0], [0, -80, 0, 0, 0, 0, 0, 80], [0, 0, 115, 0, 0, 0, 0, 0], [0, 123, 0, 123, 0, -123, 0, 0], [8, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 105, 0, -105, 0, -105], [0, 0, 0, 0, 0, 0, 104, 0], [0, 0, 0, -48, 0, -48, 0, 48], [0, -63, 0, 0, 0, 0, 0, -63], [-83, 0, 0, 0, 83, 0, 0, 0], [0, 0, 0, -56, 0, -56, 0, 56], [0, 0, 0, 0, 155, 0, 0, 0], [0, 0, -10, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 53, 0], [0, 0, 0, 96, 0, -96, 0, -96], [0, -88, 0, 0, 0, 0, 0, -88], [0, -203, 0, 203, 0, 203, 0, 0], [113, 0, 0, 0, 0, 0, 0, 0], [0, 96, 0, 0, 0, 0, 0, -96], [0, 0, -73, 0, 0, 0, 0, 0], [0, 152, 0, 152, 0, -152, 0, 0], [123, 0, 0, 0, 0, 0, 0, 0], [-261, 0, 0, 0, 261, 0, 0, 0], [0, 0, -56, 0, 0, 0, 56, 0], [0, 299, 0, 0, 0, 0, 0, 299], [-40, 0, 0, 0, 40, 0, 0, 0], [0, 64, 0, 64, 0, -64, 0, 0], [0, 0, 0, -163, 0, -163, 0, 163], [0, 0, 0, 0, 209, 0, 0, 0], [0, -147, 0, 147, 0, 147, 0, 0], [0, 0, 0, 312, 0, -312, 0, -312], [0, 136, 0, 0, 0, 0, 0, 136], [0, 288, 0, -288, 0, -288, 0, 0], [-101, 0, 0, 0, 0, 0, 0, 0], [0, 0, -184, 0, 0, 0, 184, 0], [0, -144, 0, 0, 0, 0, 0, 144], [0, 0, 544, 0, 0, 0, 0, 0], [0, 0, 0, 57, 0, -57, 0, -57], [0, 0, 0, 0, 0, 0, -282, 0], [0, 0, 0, 216, 0, 216, 0, -216], [0, 0, -191, 0, 0, 0, 191, 0], [0, 176, 0, 0, 0, 0, 0, 176], [0, 0, 0, 0, -157, 0, 0, 0], [0, 0, 193, 0, 0, 0, 0, 0], [0, 0, 0, -237, 0, 237, 0, 237], [0, 0, 0, 0, 343, 0, 0, 0], [0, -160, 0, 0, 0, 0, 0, -160], [0, 291, 0, -291, 0, -291, 0, 0], [0, 0, 461, 0, 0, 0, -461, 0], [0, 0, -159, 0, 0, 0, 0, 0], [37, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -328, 0, 328, 0, 328], [0, -32, 0, 0, 0, 0, 0, 32], [0, 0, 0, 0, 0, 0, -40, 0], [0, 0, 0, 449, 0, 449, 0, -449], [-769, 0, 0, 0, 769, 0, 0, 0], [0, 0, 0, 344, 0, 344, 0, -344], [0, 0, 0, 0, 167, 0, 0, 0], [0, -299, 0, 299, 0, 299, 0, 0], [0, 0, 0, 0, 0, 0, -32, 0], [0, 0, 0, 0, -648, 0, 0, 0], [0, 312, 0, 0, 0, 0, 0, 312], [0, 72, 0, -72, 0, -72, 0, 0], [0, 0, 368, 0, 0, 0, -368, 0], [0, 312, 0, 0, 0, 0, 0, -312], [0, 0, 69, 0, 0, 0, 0, 0], [0, -336, 0, -336, 0, 336, 0, 0], [0, -389, 0, 0, 0, 0, 0, 389], [0, 0, 0, 0, 0, 0, 507, 0], [0, 0, 0, -21, 0, -21, 0, 21], [474, 0, 0, 0, -474, 0, 0, 0], [0, 117, 0, 117, 0, -117, 0, 0], [0, 0, 0, -393, 0, -393, 0, 393], [0, -199, 0, 199, 0, 199, 0, 0], [0, 0, 0, 0, 0, 0, 851, 0], [73, 0, 0, 0, 0, 0, 0, 0], [0, 0, 251, 0, 0, 0, -251, 0], [0, 312, 0, 312, 0, -312, 0, 0], [0, 0, 0, -232, 0, 232, 0, 232], [0, -704, 0, 0, 0, 0, 0, 704], [35, 0, 0, 0, -35, 0, 0, 0], [0, 0, 0, 0, 0, 0, -337, 0], [0, 333, 0, 0, 0, 0, 0, 333], [0, -471, 0, -471, 0, 471, 0, 0], [0, 0, 0, 483, 0, 483, 0, -483], [0, 0, 0, 0, -608, 0, 0, 0], [0, 0, -421, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -761, 0], [0, 0, 0, 71, 0, -71, 0, -71], [0, 0, 0, 0, 941, 0, 0, 0], [-805, 0, 0, 0, 0, 0, 0, 0], [0, 56, 0, 0, 0, 0, 0, -56], [0, 0, 1104, 0, 0, 0, 0, 0], [0, 531, 0, 531, 0, -531, 0, 0], [0, 0, 0, -104, 0, 104, 0, 104], [0, 704, 0, 0, 0, 0, 0, -704], [1249, 0, 0, 0, -1249, 0, 0, 0], [0, 0, 849, 0, 0, 0, -849, 0], [0, -400, 0, 0, 0, 0, 0, -400], [0, 83, 0, 83, 0, -83, 0, 0], [0, 0, 0, 0, -16, 0, 0, 0], [0, -719, 0, 719, 0, 719, 0, 0], [0, 0, 0, 0, -231, 0, 0, 0], [0, -85, 0, 85, 0, 85, 0, 0], [0, 0, -1080, 0, 0, 0, 1080, 0], [0, 0, -344, 0, 0, 0, 0, 0], [-640, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [301, 0, 0, 0, -301, 0, 0, 0], [0, 0, 0, 0, 0, 0, -367, 0], [0, 0, 0, 57, 0, 57, 0, -57], [417, 0, 0, 0, -417, 0, 0, 0], [0, 207, 0, 207, 0, -207, 0, 0], [0, 0, 379, 0, 0, 0, 0, 0], [0, 0, 0, 488, 0, -488, 0, -488], [0, 456, 0, -456, 0, -456, 0, 0], [64, 0, 0, 0, 0, 0, 0, 0], [0, 136, 0, 0, 0, 0, 0, -136], [0, 0, -339, 0, 0, 0, 0, 0], [0, -512, 0, -512, 0, 512, 0, 0], [639, 0, 0, 0, 0, 0, 0, 0], [0, 200, 0, 0, 0, 0, 0, -200], [0, 0, 959, 0, 0, 0, -959, 0], [0, -129, 0, 0, 0, 0, 0, -129], [-1147, 0, 0, 0, 1147, 0, 0, 0], [0, 528, 0, 528, 0, -528, 0, 0], [0, 0, -655, 0, 0, 0, 0, 0], [0, 225, 0, -225, 0, -225, 0, 0], [0, 0, 0, 0, 0, 0, 261, 0], [0, 0, 0, 552, 0, -552, 0, -552], [0, 0, 525, 0, 0, 0, -525, 0], [0, -360, 0, 0, 0, 0, 0, 360], [1272, 0, 0, 0, 0, 0, 0, 0], [0, -393, 0, 0, 0, 0, 0, 393], [0, 0, 0, 0, 0, 0, -1265, 0], [0, 0, 0, 799, 0, 799, 0, -799], [0, -552, 0, 0, 0, 0, 0, -552], [0, 328, 0, 328, 0, -328, 0, 0], [0, 0, -1147, 0, 0, 0, 0, 0], [0, -576, 0, 576, 0, 576, 0, 0], [0, 0, 0, -663, 0, 663, 0, 663], [0, 965, 0, 0, 0, 0, 0, 965], [-1115, 0, 0, 0, 0, 0, 0, 0], [0, 0, -1320, 0, 0, 0, 1320, 0]]
